__author__ = "Zafer Aydin"
__date__ = "11/19/2009"
__project__ = "Protein Structure Prediction"

usage = """
        Usage: python compute_accuracy_measures.py <input fastas list filename> <n states> <prediction task switch> <output html filename> 
"""

alphabet_ss_8 = "HBEGITSL"  
alphabet_ss_3 = "HEL"
alphabet_ss_5 = "HEebL"
alphabet_torsion_5 = "OEGAB"

def give_me_ss_number_3_state( ss ):

        if (ss == 'H'):

                return 0

        elif (ss == 'E'):

                return 1

        elif (ss == 'L'):

                return 2

        else:
                return -1

def give_me_ss_number_5_state( ss ):

        if (ss == 'H'):

                return 0

        elif (ss == 'E'):

                return 1

        elif (ss == 'e'):

                return 2

        elif (ss == 'b'):

                return 3

        elif (ss == 'L'):

                return 4

        else:
                return -1

def give_me_torsion_number_5_state( torsion ):

        if (torsion == 'O'):

                return 0

        elif ((torsion == 'R') or (torsion == 'E')):

                return 1

        elif (torsion == 'G'):

                return 2

        elif (torsion == 'A'):

                return 3

        elif (torsion == 'B'):

                return 4

        else:
                return -1

def accumulate_expressions_for_sov_measures(true_label_numbers, pred_label_numbers, n_states, SOVs):

	n_labels = len(true_label_numbers)

	done = 0
	i=0

	#print true_label_numbers
	#print pred_label_numbers

	while not done:

		true_label_number = true_label_numbers[i]
		pred_label_number = pred_label_numbers[i]

		if (true_label_number == pred_label_number):

			#Found an overlapping segment
			
			#Find the segment start and end positions for the observed sequence
			k=i	
			while (true_label_numbers[k] == true_label_number):
				
				if (k == 0):
					k -= 1
					break
				else:
					k -= 1
					continue				

			segment_start_index_obs = k+1

			k=i
                        while (true_label_numbers[k] == true_label_number):

                                if (k == n_labels-1):
					k += 1
                                        break
                                else:
					k += 1
                                        continue

                        segment_end_index_obs = k-1

			#Find the segment start and end positions for the predicted sequence
                        k=i
                        while (pred_label_numbers[k] == pred_label_number):

                                if (k == 0):
                                        k -= 1
                                        break
                                else:
                                        k -= 1
                                        continue

                        segment_start_index_pred = k+1

                        k=i
                        while (pred_label_numbers[k] == pred_label_number):

                                if (k == n_labels-1):
                                        k += 1
                                        break
                                else:
                                        k += 1
                                        continue
                        
                        segment_end_index_pred = k-1

			segment_length_obs = segment_end_index_obs - segment_start_index_obs + 1
			segment_length_pred = segment_end_index_pred - segment_start_index_pred + 1
	
			minimum_label_index_left = segment_start_index_obs
			if (segment_start_index_pred < minimum_label_index_left):
				minimum_label_index_left = segment_start_index_pred

			maximum_label_index_left = segment_start_index_obs
                        if (segment_start_index_pred > maximum_label_index_left):
                                maximum_label_index_left = segment_start_index_pred

			maximum_label_index_right = segment_end_index_obs
			if (segment_end_index_pred > maximum_label_index_right):
				maximum_label_index_right = segment_end_index_pred

                        minimum_label_index_right = segment_end_index_obs
                        if (segment_end_index_pred < minimum_label_index_right):
                                minimum_label_index_right = segment_end_index_pred

			maxov = maximum_label_index_right - minimum_label_index_left + 1

			minov = 0
			for j in range(maximum_label_index_left, minimum_label_index_right+1, 1):
				if (true_label_numbers[j] == pred_label_numbers[j]):		
					minov += 1

                        delta_value_candidates = [0]*4
                        delta_value_candidates[0] = maxov-minov
                        delta_value_candidates[1] = minov
                        delta_value_candidates[2] = int(segment_length_obs / 2.0)
                        delta_value_candidates[3] = int(segment_length_pred / 2.0)

                        delta_value = min(delta_value_candidates)
                        expresion_to_sum = float((minov + delta_value)*segment_length_obs) / float(maxov)
			expression_to_check = (minov + delta_value) / maxov

			#print expression_to_check
			#print expresion_to_sum

                        #print 'minov'
                        #print minov
                        #print maxov
                        #print delta_value
			#print segment_length_obs
			#print segment_start_index_obs
			#print segment_end_index_obs
			#print segment_start_index_pred
			#print segment_end_index_pred

                        SOVs[true_label_number] += expresion_to_sum
                        SOVs[n_states] += expresion_to_sum
		
			if (segment_end_index_obs == n_labels-1):

				done = 1
				continue
			else:
				i = segment_end_index_obs + 1
		else:
                        if (i == n_labels-1):

                                done = 1
                                continue
                        else:
                                i += 1
	return SOVs

import sys
import math
import string

if __name__ == "__main__":

        if ( len(sys.argv) != 5 ):

                sys.stderr.write(usage)
                sys.exit(-1)

	fastas_list_filename = sys.argv[1]
	prediction_task = int(sys.argv[2])
        n_states = int(sys.argv[3])
	output_filename = sys.argv[4]

	confusion_matrix = [0]*n_states
        for i in range(n_states):
        	confusion_matrix[i] = [0]*n_states

        row_sums = [0]*n_states
        col_sums = [0]*n_states
        total_sum = 0
        precisions = [0]*(n_states+1)
	recalls = [0]*(n_states+1)

	true_positives = [0]*n_states
	true_negatives = [0]*n_states 
	false_positives = [0]*n_states
	false_negatives = [0]*n_states
	MCCs = [0]*n_states
	SOVs = [0]*(n_states+1)

	fastas_list_file = open(fastas_list_filename, 'r')

	for line in fastas_list_file:

		fasta_filename = line.rstrip()
		out_filename = fasta_filename.replace('.fasta', '.out')

		out_file = open(out_filename, 'r')
		out_file_inside = out_file.readlines()		
		out_file.close()
		
		n_aas = len(out_file_inside)

		true_label_numbers = []
		pred_label_numbers = []

		for i in range(n_aas):

			token_list = out_file_inside[i].split()
			true_label = token_list[2]
			pred_label = token_list[3]
		
 			if (prediction_task == 1): #3 state SS prediction
		
				true_label_number = give_me_ss_number_3_state(true_label)
				pred_label_number = give_me_ss_number_3_state(pred_label)
		
			elif (prediction_task == 2): #5 state SS prediction

                                true_label_number = give_me_ss_number_5_state(true_label)
                                pred_label_number = give_me_ss_number_5_state(pred_label)
				
			elif (prediction_task == 3): #5 state Torsion Class prediction

                                true_label_number = give_me_torsion_number_5_state(true_label)
                                pred_label_number = give_me_torsion_number_5_state(pred_label)

			else:
				print 'Which prediction task did you mean? Sorry didnt understand your choice. Exiting.'
				sys.exit(-1)

			true_label_numbers += [ true_label_number ]
			pred_label_numbers += [ pred_label_number ]
	
			confusion_matrix[true_label_number][pred_label_number] += 1

		SOVs = accumulate_expressions_for_sov_measures(true_label_numbers, pred_label_numbers, n_states, SOVs)
		
	fastas_list_file.close()

	for i in range(n_states):
	
		for j in range(n_states):

			row_sums[i] += confusion_matrix[i][j]
			col_sums[j] += confusion_matrix[i][j]

	total_sum = sum(row_sums)
	total_correct_preds = 0

	for i in range(n_states):

		if (col_sums[i] != 0):

			precisions[i] = 100.0 * confusion_matrix[i][i] / float(col_sums[i])
		else:
			precisions[i] = 0.0

		if (row_sums[i] != 0):

			print SOVs[i]
			print row_sums[i]

			recalls[i] = 100.0 * confusion_matrix[i][i] / float(row_sums[i])
			SOVs[i] = 100.0 * SOVs[i] / float(row_sums[i])
		else:
			recalls[i] = 0.0
			SOVs[i] = 0.0

		total_correct_preds += confusion_matrix[i][i]

	#This is the overall accuracy
	if (total_sum != 0):
		precisions[n_states] = 100.0 * total_correct_preds / float(total_sum)
		recalls[n_states] = 100.0 * total_correct_preds / float(total_sum)
		SOVs[n_states] = 100.0 * SOVs[n_states] / float(total_sum)
	else:
		precisions[n_states] = 0.0
		recalls[n_states] = 0.0
		SOVs[n_states] = 0.0

	#Matthews Correlation Coefficients for all states
	for k in range(n_states): # this loops over possible Mathhew's correlation coefficients that we compute (for all states)

	        for i in range(n_states):

        	        for j in range(n_states):

				if ((i == k) and (j == k)):

					true_positives[k] += confusion_matrix[i][j]

				elif ((i == k) and (j != k)):

					false_negatives[k] += confusion_matrix[i][j]

				elif ((i != k) and (j == k)):

                                        false_positives[k] += confusion_matrix[i][j]

				elif ((i != k) and (j != k)):

                                        true_negatives[k] += confusion_matrix[i][j]

	for k in range(n_states):

		denominator = (true_negatives[k]+false_negatives[k])*(true_negatives[k]+false_positives[k])*(true_positives[k]+false_negatives[k])*(true_positives[k]+false_positives[k])
		MCCs[k] = (true_positives[k]*true_negatives[k]-false_positives[k]*false_negatives[k]) / math.sqrt(denominator)
	
	#Generate the html table now
	output_file = open(output_filename, 'w')
        output_file.write('<html>\n')
        output_file.write('<body>\n')
        output_file.write('<blockquote>\n')
        output_file.write('<TABLE align=\"centre\"border=\"1\">\n')
	output_file.write('<CAPTION><EM>Confusion Matrix and Accuracy Measures</EM></CAPTION>\n')

	if (prediction_task == 1):
		
		output_file.write('<TR><TH>True Labels</TH><TH>Pred=H</TH><TH>Pred=E</TH><TH>Pred=L</TH><TH>Row Sums</TH><TH>Precision(%)</TH><TH>Recall(%)</TH><TH>SOV(%)</TH><TH>MCC</TH>\n')

	elif (prediction_task == 2):
        
                output_file.write('<TR><TH>True Labels</TH><TH>Pred=H</TH><TH>Pred=E</TH><TH>Pred=e</TH><TH>Pred=b</TH><TH>Pred=L</TH><TH>Row Sums</TH><TH>Precision(%)</TH><TH>Recall(%)</TH><TH>SOV(%)</TH><TH>MCC</TH>\n')

	elif (prediction_task == 3):

		output_file.write('<TR><TH>True Labels</TH><TH>Pred=O</TH><TH>Pred=R</TH><TH>Pred=G</TH><TH>Pred=A</TH><TH>Pred=B</TH><TH>Row Sums</TH><TH>Precision(%)</TH><TH>Recall(%)</TH><TH>SOV(%)</TH><TH>MCC</TH>\n')

	for i in range(n_states):

		if (prediction_task == 1):

			label = alphabet_ss_3[i]
			output_file.write('<TR><TH>True=%s</TH>' % label)

		elif (prediction_task == 2):
	
			label = alphabet_ss_5[i]
                        output_file.write('<TR><TH>True=%s</TH>' % label)

		elif (prediction_task == 3):

                        label = alphabet_torsion_5[i]
                        output_file.write('<TR><TH>True=%s</TH>' % label)

		for j in range(n_states):

			output_file.write('<TD><center>%d</TD>' % confusion_matrix[i][j])

		output_file.write('<TD><center>%d</TD>' % row_sums[i])
		output_file.write('<TD><center>%.2f</TD>' % precisions[i])
		output_file.write('<TD><center>%.2f</TD>' % recalls[i])
		output_file.write('<TD><center>%.2f</TD>' % SOVs[i])
		output_file.write('<TD><center>%.2f</TD>' % MCCs[i])
		output_file.write('\n')

	output_file.write('<TR><TH>Column Sums</TH>')
	
	for i in range(n_states):
	
		output_file.write('<TD><center>%d</TD>' % col_sums[i])

	output_file.write('<TD><center>%d</TD>' % total_sum)
	output_file.write('<TD><center>%.2f</TD>' % precisions[n_states])
	output_file.write('<TD><center>%.2f</TD>' % recalls[n_states])
	output_file.write('<TD><center>%.2f</TD>' % SOVs[n_states])
	output_file.write('<TD><center>%s</TD>' % '---')
	output_file.write('\n')
        output_file.write('</TABLE>\n')

        output_file.write('</blockquote>\n')
        output_file.write('</body>\n')
        output_file.write('</html>\n')
		
	output_file.close()
	fastas_list_file.close()

