__author__ = "Zafer Aydin"
__date__ = "04/08/2009"

# FILENAME: divide_observation_files_wrt_label_tuples.py
# PROJECT: Protein Secondary Structure Prediction

usage="""
	python divide_observation_files_wrt_label_tuples.py <dataset relative dir> <dataset filename> <number of classes (label alphabet size)> <laa> <lss> <output files root>
"""

import sys
import math

if __name__ == "__main__":


	if (len(sys.argv) != 7):

		sys.stderr.write(usage)
		sys.exit(-1)

	dataset_relative_path = sys.argv[1]
	training_set_filename = sys.argv[2]
	n_classes = int(sys.argv[3])
	laa = int(sys.argv[4])
	lss = int(sys.argv[5])
	output_files_root = sys.argv[6]

	training_set_file = open(dataset_relative_path + training_set_filename, 'r')

	done = 0
	n_ss_tuples = pow(n_classes, lss+1)

	for i in range(n_ss_tuples):

        	tuple_filename = dataset_relative_path + output_files_root + '_laa_' + str(laa) + '_lss_' + str(lss) + '_tuple_' + str(i) + '.obs'
                tuple_file = open(tuple_filename, 'w')
                tuple_file.close()

	while not done:

		an_obs_filename = training_set_file.readline()

		if (len(an_obs_filename) == 0):

			done = 1
			continue

		an_obs_filename = an_obs_filename.rstrip()
		an_obs_file = open(an_obs_filename, 'r')
		obs_file_inside = an_obs_file.readlines()

		n_frames = len(obs_file_inside)				
	
		if (laa > lss):

			safe_window_size = laa+laa #We add an additional laa here because the obs files have laa artificial profile vectors which should also be skipped

		else:
			safe_window_size = lss+laa #We add an additional laa here because the obs files have laa artificial profile vectors which should also be skipped			

		for i in range(n_frames-safe_window_size):

			tuple_index = 0
			
			for j in range(lss+1):

				a_frame = obs_file_inside[i+safe_window_size-j].rstrip()
				token_list = a_frame.split()
				tuple_index += pow(n_classes,j)*int(token_list[-3])
			
			frames = []
			for j in range(laa+1):

				a_frame = obs_file_inside[i+safe_window_size-j].rstrip()
				token_list = a_frame.split()
				
				for k in range(len(token_list)-3): #Do not include the last 3 values which are the secondary structure label Di (state count down var), and Fi (change state var)  

					frames += [ token_list[k] ]				

			
			#tuple_filename = dataset_relative_path + output_files_root + '_laa_' + str(laa) + '_lss_' + str(lss) + '_tuple_' + str(tuple_index) + '.obs'			
			tuple_filename = dataset_relative_path + output_files_root + '_ss_tuple_' + str(tuple_index) + '.obs'
			tuple_file = open(tuple_filename, 'a')

			tuple_filename2 = dataset_relative_path + output_files_root + '.obs'
                        tuple_file2 = open(tuple_filename2, 'a')

			list_filename = dataset_relative_path + output_files_root + '_ss_tuple_' + str(tuple_index) + '.list'
                        list_file = open(list_filename, 'w')
                        list_file.write(tuple_filename)
			list_file.write('\n')
                        list_file.close()


			for j in range(len(frames)):

				tuple_file.write('%.7f ' % float(frames[j]))
				tuple_file2.write('%.7f ' % float(frames[j]))

			tuple_file.write('\n')
	                tuple_file.close()
			tuple_file2.write('\n')
                        tuple_file2.close()

		an_obs_file.close()

	list_filename2 = dataset_relative_path + output_files_root + '.list'
        list_file2 = open(list_filename2, 'w')
        list_file2.write(tuple_filename2)
	list_file2.write('\n')
        list_file2.close()

	training_set_file.close() 

