__author__ = "Zafer Aydin"
__date__ = "12/29/2009"
__project__ = "Protein Structure Prediction"

usage = """
	Usage: python generate_out_files.py <prediction_task> <number_of_state_classes> <pssm_window_size> <dataset_dir> <fastas all list filename> <obs files list filename> <postp files list filename model>"
"""

import sys

ss_alphabet_3 = "HEL"
ss_alphabet_5 = "HEebL"
ss_alphabet_8 = "HBEGITSLLL"   #Label 9 and 10 are also L in SS labels
torsion_alphabet_5 = "ORGAB"

def generate_true_ss_file(prediction_task, data_dir, obs_files_list_filename, pssm_window_size, true_ss_filename):

	obs_files_list_file = open( obs_files_list_filename, 'r' )
	true_ss_file = open( true_ss_filename, 'w' )

	for line in obs_files_list_file:

		obs_filename = line.rstrip()
		obs_file = open( obs_filename, 'r' )
		
		true_ss_labels_str = ''
		line_counter = 0

		for obs_line in obs_file:
			
			if (line_counter < pssm_window_size):
				line_counter += 1
				continue
			
			token_list = obs_line.split()	
			#ss_label_int = int( token_list[-1] )
			ss_label_int = int( token_list[-3] )

			if (prediction_task == 1):

				true_ss_labels_str += ss_alphabet_3[ss_label_int]  
		
			elif (prediction_task == 2):

				true_ss_labels_str += ss_alphabet_5[ss_label_int]

			elif (prediction_task == 3):
			
				true_ss_labels_str += torsion_alphabet_5[ss_label_int]
	
		obs_file.close()

		protein_id = obs_filename.replace(data_dir, '')
		protein_id = protein_id.replace('.obs', '')

		true_ss_file.write( '>' + protein_id + '\n')
		true_ss_file.write(true_ss_labels_str + '\n');

	true_ss_file.close()
	obs_files_list_file.close()


def generate_true_and_predicted_ss_file( prediction_task, number_of_state_classes, data_dir, pssm_window_size, fasta_filenames_list_filename, obs_files_list_filename, postp_files_list_filename ):

        obs_files_list_file = open( obs_files_list_filename, 'r' )
	postp_files_list_file = open( postp_files_list_filename, 'r' )
	fasta_filenames_file = open( fasta_filenames_list_filename, 'r')

	done  = 0

	while not done:
		
		line = postp_files_list_file.readline()
	
		if ( len( line ) == 0 ):

			done = 1
			continue

                line_fasta_filename = fasta_filenames_file.readline().rstrip()
                fasta_filename = data_dir + line_fasta_filename
                fasta_file = open(fasta_filename, 'r')
                fasta_id = fasta_file.readline()
                aa_seq = fasta_file.readline().rstrip()
                fasta_file.close()

		postp_filename = line.rstrip()

		postp_file = open( postp_filename, 'r' )

                if ('nc.aapw.test.postp.sigmoid' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('nc.aapw.test.postp.sigmoid', 'out')

                elif ('cn.aapw.test.postp.sigmoid' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('cn.aapw.test.postp.sigmoid', 'out')

                elif ('nc.aapw.test.postp.linear' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('nc.aapw.test.postp.linear', 'out')

                elif ('cn.aapw.test.postp.linear' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('cn.aapw.test.postp.linear', 'out')

		elif ('nc.dcw.test.postp.sigmoid' in postp_filename):

			postp_file_out_filename = postp_filename.replace('nc.dcw.test.postp.sigmoid', 'out')

		elif ('cn.dcw.test.postp.sigmoid' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('cn.dcw.test.postp.sigmoid', 'out')

		elif ('nc.dcw.test.postp.linear' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('nc.dcw.test.postp.linear', 'out')

		elif ('cn.dcw.test.postp.linear' in postp_filename):

                        postp_file_out_filename = postp_filename.replace('cn.dcw.test.postp.linear', 'out')

		postp_file_out = open( postp_file_out_filename, 'w')

		pred_ss_labels_str = ''

		line_counter = 1

		postp_file_inside =  postp_file.readlines()

		postp_file_inside_length = len(postp_file_inside)

		#Read the TRUE labels 
		obs_filename = obs_files_list_file.readline().rstrip()
                obs_file = open( obs_filename, 'r' )

                true_labels = ''
                line_counter = 0

                for obs_line in obs_file:

                        if (line_counter < pssm_window_size):
                                line_counter += 1
                                continue

                        token_list = obs_line.split()
                        #ss_label_int = int( token_list[-1] )
                        ss_label_int = int( token_list[-3] )

                        #print ss_label_int

			if (prediction_task == 1):

                                true_labels += ss_alphabet_3[ss_label_int]

                        elif (prediction_task == 2):

                                true_labels += ss_alphabet_5[ss_label_int]

                        elif (prediction_task == 3):

                                true_labels += torsion_alphabet_5[ss_label_int]

                obs_file.close()

		for i in range(0, postp_file_inside_length, 1):

			postp_line = postp_file_inside[i]
			token_list = postp_line.split()
			
			#print '%s' % token_list

			#print len( token_list )
			#print number_of_state_classes

			if ( len( token_list ) == number_of_state_classes ):

				max_value = -1000
				max_index = 0			

				postp_values = []
	
				for j in range(0, len( token_list ), 1):
				
					#Postp value is the average of postp values from the forward and reverse models
					#postp_value = ( float( token_list[j] ) + float( token_list_reverse[j] ) ) / 2.0
					postp_value = float(token_list[j])
					#postp_value = float(token_list_reverse[j])

					postp_values += [ postp_value ]

					#print '%f' % float(token_list[j])
					#print '%f' % float(token_list_reverse[j])
					#print '%f' % postp_value

					if ( postp_value > max_value):
						
						max_value = postp_value
						max_index = j


				#print postp_values
				
				#print '%s' % ssalphabet[max_index]

				if (prediction_task == 1):
			 	
					pred_label = ss_alphabet_3[max_index]
					pred_ss_labels_str += pred_label
					
				elif (prediction_task == 2):

					pred_label = ss_alphabet_5[max_index]
	                                pred_ss_labels_str += pred_label
	
				elif (prediction_task == 3):

					pred_label = torsion_alphabet_5[max_index]
					pred_ss_labels_str += pred_label
			
				aa_type = aa_seq[i]
				true_label = true_labels[i]

				if (pred_label == 'R'):
					pred_label = 'E'

				kk=i+1
                                postp_file_out.write("%d %s %s %s " % (kk, aa_type, true_label, pred_label))

				for j in range(len(token_list)):

					postp_file_out.write("%f\t" % postp_values[j])

				postp_file_out.write("\n")

				#print max_index	
				#print pred_ss_labels_str

			#print pred_ss_labels_str
			line_counter += 1

		postp_file.close()
		postp_file_out.close()

	obs_files_list_file.close()
	postp_files_list_file.close()
	fasta_filenames_file.close()


#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# main program is here

import sys
import math

if __name__ == "__main__":
    
	if ( len(sys.argv) != 8 ):
		
		sys.stderr.write(usage)
		sys.exit(-1)
	
	prediction_task = int(sys.argv[1])
	number_of_state_classes = int(sys.argv[2])
	data_dir = sys.argv[3]
	fasta_filenames_list_filename = sys.argv[4]
	obs_files_list_filename = sys.argv[5]
	postp_files_list_filename = sys.argv[6]
	pssm_window_size = int(sys.argv[7])

	generate_true_and_predicted_ss_file( prediction_task, number_of_state_classes, data_dir, pssm_window_size, fasta_filenames_list_filename, obs_files_list_filename, postp_files_list_filename ) 
	


 


