#!/bin/bash

if [ $# -ne 33 ]
then
        echo ""
        echo "Usage: $0 <prediction_task> <data_dir> <models_dir> <scripts_dir> <gmtk_bins_dir> <matlab_dir> <train_str_filename> <train_master_filename_in> <train_master_filename_out> <non_trainable_list_filename> <test_str_filename> <tri_filename> <test_master_filename_in> <dcw_train_root> <dcw_test_root> <dcw_test_postp_root> <n_cv_folds> <n_states> <pssm_window_size> <n_continuous_features> <n_discrete_features> <n_gc> <l2_reg_thr_mean> <l2_reg_thr_dlink> <lldp_thr> <varfloor_thr> <max_em_iters> <learned_means_filename> <learned_diag_covs_filename> <learned_dlinks_filename> <learned_dense_cpts_filename> <diag_cov_weight_filename> <forward_reverse_string>" 
        exit 1
fi

prediction_task=$1
data_dir=$2
models_dir=$3
scripts_dir=$4
gmtk_bins_dir=$5
matlab_dir=$6
train_str_filename=$7
train_master_filename_in=$8
train_master_filename_out=$9
non_trainable_list_filename=${10}
test_str_filename=${11}
tri_filename=${12}
test_master_filename_in=${13}
dcw_train_root=${14}
dcw_test_root=${15}
dcw_test_postp_root=${16}
n_cv_folds=${17}
n_states=${18}
pssm_window_size=${19}
n_continuous_features=${20}
n_discrete_features=${21}
n_gc=${22}
l2_reg_thr_mean=${23}
l2_reg_thr_dlink=${24}
lldp_thr=${25}
varfloor_thr=${26}
max_em_iters=${27}
learned_means_filename=${28}
learned_diag_covs_filename=${29}
learned_dlinks_filename=${30}
learned_dense_cpts_filename=${31}
diag_cov_weight_filename=${32}
forward_reverse_string=${33}

n_skip=$pssm_window_size
accuracy_filename="q3.txt"
n_dim=$n_continuous_features

echo "Doing cross-validated training for parameter optimization..."
date

echo $dcw_train_root

for ((fold_index = 0; fold_index < n_cv_folds; fold_index++))
#for ((fold_index = 0; fold_index < 0; fold_index++))
do
	echo "fold index = $fold_index"

	${scripts_dir}runEM_cv.sh $gmtk_bins_dir $train_str_filename $train_master_filename_in $train_master_filename_out $data_dir$dcw_train_root $non_trainable_list_filename $fold_index $n_continuous_features $n_discrete_features $n_skip $l2_reg_thr_mean $l2_reg_thr_dlink $lldp_thr $varfloor_thr $max_em_iters >& ./runEM.$forward_reverse_string.$fold_index.log

	\cp ${models_dir}$learned_means_filename ${models_dir}$learned_means_filename.$fold_index.$forward_reverse_string
	\cp ${models_dir}$learned_diag_covs_filename ${models_dir}$learned_diag_covs_filename.$fold_index.$forward_reverse_string
	\cp ${models_dir}$learned_dlinks_filename ${models_dir}$learned_dlinks_filename.$fold_index.$forward_reverse_string
	\cp ${models_dir}$learned_dense_cpts_filename ${models_dir}$learned_dense_cpts_filename.$fold_index.$forward_reverse_string
done

##\cp ${data_dir}q3s.txt $models_dir
#dcw_init_values=(0.0005 0.001 0.005 0.01 0.05 0.1 0.5)
#dcw_init_values=(0.01)
#n_init_values=${#dcw_init_values[@]}

max_accuracy=0.0
max_dcw=0.0
max_accuracy_filename="max_accuracy.txt"

n_opt_reps=2

next_weights_filename="init_dcws.txt"
#next_weights_filename="next_weights.txt"
previous_increment_filename="previous_increment.txt"

\cp "init_previous_increment.txt" $previous_increment_filename

n_next_weights=`wc -l $next_weights_filename | awk '{print $1'}`

echo $n_next_weights
for ((j=0; j < n_opt_reps; j++))
#for ((j=1; j < n_opt_reps; j++))
do
   for ((i=0; i < n_next_weights; i++))
   #For ((i=0; i < 1; i++))
   do
        k=$((i+1))
	dcw=`head -$k $next_weights_filename | tail -1`
	#dcw=${dcw_init_values[$i]}

	echo $dcw

	output_html="cm_optimization.$dcw.html"

	for ((fold_index = 0; fold_index < n_cv_folds; fold_index++))
	#for ((fold_index = 0; fold_index < 0; fold_index++))
	do
                \cp ${models_dir}$learned_means_filename.$fold_index.$forward_reverse_string ${models_dir}$learned_means_filename
                \cp ${models_dir}$learned_diag_covs_filename.$fold_index.$forward_reverse_string ${models_dir}$learned_diag_covs_filename
                \cp ${models_dir}$learned_dlinks_filename.$fold_index.$forward_reverse_string ${models_dir}$learned_dlinks_filename
                \cp ${models_dir}$learned_dense_cpts_filename.$fold_index.$forward_reverse_string ${models_dir}$learned_dense_cpts_filename

		if ${matlab_dir}matlab -nodisplay -nojvm -r \
		 "add_diagonal_covariance_component_in_full_cov_form('${models_dir}$learned_means_filename', '${models_dir}$learned_diag_covs_filename', '${models_dir}$learned_dlinks_filename', '$n_gc', '$n_dim', '$pssm_window_size', '$dcw'); exit"
        	then
                	echo "Added a diagonal covariance component to the full covariance form"
	        else
        	        echo "[Error] Could not add a diagonal covariance component"
                	exit 1
	        fi

		${scripts_dir}runJT_cv.sh $gmtk_bins_dir $test_str_filename $tri_filename $test_master_filename_in $data_dir$dcw_test_root $fold_index $n_continuous_features $n_discrete_features $n_skip >& ./runJT.$forward_reverse_string.$fold_index.log
		python ${scripts_dir}generate_postp_files.py $n_states $data_dir$dcw_test_postp_root.$fold_index.list ./runJT.$forward_reverse_string.$fold_index.log	
	done

	cat_string1=""
	cat_string2=""
	for ((fold_index = 0; fold_index < n_cv_folds; fold_index++))
        do
		cat_string1="$cat_string1 $data_dir${dcw_test_root}.$fold_index.list"
		cat_string2="$cat_string2 $data_dir${dcw_test_postp_root}.$fold_index.list"
	done

	cat $cat_string1 >& $data_dir${dcw_test_root}.all.list
	cat $cat_string2 >& $data_dir${dcw_test_postp_root}.all.list
	fastas_all_filename="fastas.all.list"
	python ${scripts_dir}convert_to_fastas_all_file.py $data_dir${dcw_test_root}.all.list $data_dir$fastas_all_filename
        python ${scripts_dir}generate_out_files.py $prediction_task $n_states $data_dir $data_dir$fastas_all_filename $data_dir$dcw_test_root.all.list $data_dir$dcw_test_postp_root.all.list $pssm_window_size
        python ${scripts_dir}compute_accuracy_measures.py $data_dir$fastas_all_filename $prediction_task $n_states $output_html

	#Get the overall accuracy value
	python ${scripts_dir}get_accuracy_value.py $output_html $accuracy_filename

	#Read the optimized weight value from file
	accuracy_value=`cat $accuracy_filename`
	echo $accuracy_value

	if [ $(echo "scale=10; if($accuracy_value > $max_accuracy) print "1" " | bc) ];
	then
		max_accuracy=$accuracy_value
		max_dcw=$dcw
		echo "max accuracy = $max_accuracy"
		echo "max dcw = $max_dcw"
	fi
   done

   next_weights_filename="next_weights.txt"
   python ${scripts_dir}get_the_next_weights_to_try.py $max_dcw $next_weights_filename $previous_increment_filename
   n_next_weights=`wc -l $next_weights_filename | awk '{print $1'}`

done

echo $max_dcw > $diag_cov_weight_filename

